@@ -87,6 +87,7 @@ class Client(object):
         random.shuffle(self.hosts)
         self.application_id = application_id
         self.api_key = api_key
+        self.timeout = urllib3.util.timeout.Timeout(connect = 1.0, read = 30.0)
         self.headers = {
             'Content-Type': 'application/json; charset=utf-8',
             'X-Algolia-API-Key': self.api_key,
@@ -134,6 +135,12 @@ class Client(object):
         """
         self.headers[key] = value
 
+    def set_timeout(self, connect_timeout, read_timeout):
+        """
+        Allow to set the connection timeout in second
+        """
+        self.timeout = urllib3.util.timeout.Timeout(connect = connect_timeout, read = read_timeout)
+
     @deprecated
     def multipleQueries(self, queries, index_name_key = "indexName"):
         return self.multiple_queries(queries, index_name_key)
@@ -150,7 +157,7 @@ class Client(object):
                     query[key] = json.dumps(query[key], cls = JSONEncoderWithDatetimeAndDefaultToString)
             requests.append({"indexName": index_name, "params": urlencode(query)})
         body = {"requests": requests}
-        return AlgoliaUtils_request(self.headers, self.hosts, "POST", "/1/indexes/*/queries", body)
+        return AlgoliaUtils_request(self.headers, self.hosts, "POST", "/1/indexes/*/queries", self.timeout, body)
 
     @deprecated
     def listIndexes(self):
@@ -162,7 +169,7 @@ class Client(object):
            {"items": [{ "name": "contacts", "created_at": "2013-01-18T15:33:13.556Z"},
                       {"name": "notes", "created_at": "2013-01-18T15:33:13.556Z"}]}
         """
-        return AlgoliaUtils_request(self.headers, self.hosts, "GET", "/1/indexes/")
+        return AlgoliaUtils_request(self.headers, self.hosts, "GET", "/1/indexes/", self.timeout)
 
     @deprecated
     def deleteIndex(self, index_name):
@@ -174,7 +181,7 @@ class Client(object):
         @param index_name the name of index to delete
         Return an object of the form {"deleted_at": "2013-01-18T15:33:13.556Z"}
         """
-        return AlgoliaUtils_request(self.headers, self.hosts, "DELETE", "/1/indexes/%s" % quote(index_name.encode('utf8'), safe=''))
+        return AlgoliaUtils_request(self.headers, self.hosts, "DELETE", "/1/indexes/%s" % quote(index_name.encode('utf8'), safe=''), self.timeout)
 
     @deprecated
     def moveIndex(self, src_index_name, dst_index_name):
@@ -187,7 +194,7 @@ class Client(object):
         @param dst_index_name the new index name that will contains a copy of src_index_name (destination will be overriten if it already exist).
         """
         request = {"operation": "move", "destination": dst_index_name}
-        return AlgoliaUtils_request(self.headers, self.hosts, "POST", "/1/indexes/%s/operation" % quote(src_index_name.encode('utf8'), safe=''), request)
+        return AlgoliaUtils_request(self.headers, self.hosts, "POST", "/1/indexes/%s/operation" % quote(src_index_name.encode('utf8'), safe=''), self.timeout, request)
 
     @deprecated
     def copyIndex(self, src_index_name, dst_index_name):
@@ -200,19 +207,24 @@ class Client(object):
         @param dst_index_name the new index name that will contains a copy of src_index_name (destination will be overriten if it already exist).
         """
         request = {"operation": "copy", "destination": dst_index_name}
-        return AlgoliaUtils_request(self.headers, self.hosts, "POST", "/1/indexes/%s/operation" % (quote(src_index_name.encode('utf8'), safe='')), request)
+        return AlgoliaUtils_request(self.headers, self.hosts, "POST", "/1/indexes/%s/operation" % (quote(src_index_name.encode('utf8'), safe='')), self.timeout, request)
 
     @deprecated
-    def getLogs(self, offset = 0, length = 10, only_errors = False):
-        return self.get_logs(offset, length, only_errors)
-    def get_logs(self, offset = 0, length = 10, only_errors = False):
+    def getLogs(self, offset = 0, length = 10, type = "all"):
+        return self.get_logs(offset, length, type)
+    def get_logs(self, offset = 0, length = 10, type = "all"):
         """
         Return last logs entries.
 
         @param offset Specify the first entry to retrieve (0-based, 0 is the most recent log entry).
         @param length Specify the maximum number of entries to retrieve starting at offset. Maximum allowed value: 1000.
         """
-        return AlgoliaUtils_request(self.headers, self.hosts, "GET", "/1/logs?offset=%d&length=%d&only_errors=%s" % (offset, length, only_errors))
+        if isinstance(type, bool):
+          if type:
+            type = "error"
+          else:
+            type = "all"
+        return AlgoliaUtils_request(self.headers, self.hosts, "GET", "/1/logs?offset=%d&length=%d&type=%s" % (offset, length, type), self.timeout)
 
     @deprecated
     def initIndex(self, index_name):
@@ -232,7 +244,7 @@ class Client(object):
         """
         List all existing user keys with their associated ACLs
         """
-        return AlgoliaUtils_request(self.headers, self.hosts, "GET", "/1/keys")
+        return AlgoliaUtils_request(self.headers, self.hosts, "GET", "/1/keys", self.timeout)
 
     @deprecated
     def getUserKeyACL(self, key):
@@ -241,7 +253,7 @@ class Client(object):
         """"
         Get ACL of a user key
         """
-        return AlgoliaUtils_request(self.headers, self.hosts, "GET", "/1/keys/%s" % key)
+        return AlgoliaUtils_request(self.headers, self.hosts, "GET", "/1/keys/%s" % key, self.timeout)
 
     @deprecated
     def deleteUserKey(self, key):
@@ -250,7 +262,7 @@ class Client(object):
         """
         Delete an existing user key
         """
-        return AlgoliaUtils_request(self.headers, self.hosts, "DELETE", "/1/keys/%s" % key)
+        return AlgoliaUtils_request(self.headers, self.hosts, "DELETE", "/1/keys/%s" % key, self.timeout)
 
     @deprecated
     def addUserKey(self, acls, validity = 0, max_queries_per_ip_per_hour = 0, max_hits_per_query = 0, indexes = None):
@@ -275,7 +287,7 @@ class Client(object):
         params = {"acl": acls, "validity": validity, "maxQueriesPerIPPerHour": max_queries_per_ip_per_hour, "maxHitsPerQuery": max_hits_per_query}
         if not indexes is None:
             params['indexes'] = indexes
-        return AlgoliaUtils_request(self.headers, self.hosts, "POST", "/1/keys", params)
+        return AlgoliaUtils_request(self.headers, self.hosts, "POST", "/1/keys", self.timeout, params)
 
     def update_user_key(self, key, acls, validity = 0, max_queries_per_ip_per_hour = 0, max_hits_per_query = 0, indexes = None):
         """
@@ -297,7 +309,7 @@ class Client(object):
         params = {"acl": acls, "validity": validity, "maxQueriesPerIPPerHour": max_queries_per_ip_per_hour, "maxHitsPerQuery": max_hits_per_query}
         if not indexes is None:
             params['indexes'] = indexes
-        return AlgoliaUtils_request(self.headers, self.hosts, "PUT", "/1/keys/" + key, params)
+        return AlgoliaUtils_request(self.headers, self.hosts, "PUT", "/1/keys/" + key, self.timeout, params)
 
     @deprecated
     def generateSecuredApiKey(self, private_api_key, tag_filters, user_token = None):
@@ -339,9 +351,9 @@ class Index(object):
           (if the attribute already exist the old object will be overwrite)
         """
         if object_id is None:
-            return AlgoliaUtils_request(self.client.headers, self.hosts, "POST", "/1/indexes/%s" % self.url_index_name, content)
+            return AlgoliaUtils_request(self.client.headers, self.hosts, "POST", "/1/indexes/%s" % self.url_index_name, self.client.timeout, content)
         else:
-            return AlgoliaUtils_request(self.client.headers, self.hosts, "PUT", "/1/indexes/%s/%s" % (self.url_index_name, quote(("%s" % object_id).encode('utf8'), safe='')), content)
+            return AlgoliaUtils_request(self.client.headers, self.hosts, "PUT", "/1/indexes/%s/%s" % (self.url_index_name, quote(("%s" % object_id).encode('utf8'), safe='')), self.client.timeout, content)
 
     @deprecated
     def addObjects(self, objects):
@@ -370,9 +382,9 @@ class Index(object):
         """
         obj_id = quote(("%s" % object_id).encode('utf8'), safe='')
         if (attributes_to_retrieve == None):
-            return AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s/%s" % (self.url_index_name, obj_id))
+            return AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s/%s" % (self.url_index_name, obj_id), self.client.timeout)
         else:
-            return AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s/%s?attributes=%s" % (self.url_index_name, obj_id, attributes_to_retrieve))
+            return AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s/%s?attributes=%s" % (self.url_index_name, obj_id, attributes_to_retrieve), self.client.timeout)
 
     @deprecated
     def getObjects(self, object_ids):
@@ -387,7 +399,7 @@ class Index(object):
         for object_id in object_ids:
             req = { "indexName" : self.index_name, "objectID": object_id}
             requests.append(req)
-        return AlgoliaUtils_request(self.client.headers, self.hosts, "POST", "/1/indexes/*/objects", { "requests" : requests});
+        return AlgoliaUtils_request(self.client.headers, self.hosts, "POST", "/1/indexes/*/objects", self.client.timeout, { "requests" : requests});
 
     @deprecated
     def partialUpdateObject(self, partial_object):
@@ -399,7 +411,7 @@ class Index(object):
         @param partial_object contains the object attributes to override, the
                object must contains an objectID attribute
         """
-        return AlgoliaUtils_request(self.client.headers, self.hosts, "POST", "/1/indexes/%s/%s/partial" % (self.url_index_name, quote(("%s" % partial_object["objectID"]).encode('utf8'), safe='')), partial_object)
+        return AlgoliaUtils_request(self.client.headers, self.hosts, "POST", "/1/indexes/%s/%s/partial" % (self.url_index_name, quote(("%s" % partial_object["objectID"]).encode('utf8'), safe='')), self.client.timeout, partial_object)
 
     @deprecated
     def partialUpdateObjects(self, objects):
@@ -425,7 +437,7 @@ class Index(object):
 
         @param object contains the object to save, the object must contains an objectID attribute
         """
-        return AlgoliaUtils_request(self.client.headers, self.hosts, "PUT", "/1/indexes/%s/%s" % (self.url_index_name, quote(("%s" % obj["objectID"]).encode('utf8'), safe='')), obj)
+        return AlgoliaUtils_request(self.client.headers, self.hosts, "PUT", "/1/indexes/%s/%s" % (self.url_index_name, quote(("%s" % obj["objectID"]).encode('utf8'), safe='')), self.client.timeout, obj)
 
     @deprecated
     def saveObjects(self, objects):
@@ -490,7 +502,7 @@ class Index(object):
         """
         if (len("%s" % object_id) == 0):
             raise AlgoliaException("object_id is required")
-        return AlgoliaUtils_request(self.client.headers, self.hosts, "DELETE", "/1/indexes/%s/%s" % (self.url_index_name, quote(("%s" % object_id).encode('utf8'), safe='')))
+        return AlgoliaUtils_request(self.client.headers, self.hosts, "DELETE", "/1/indexes/%s/%s" % (self.url_index_name, quote(("%s" % object_id).encode('utf8'), safe='')), self.client.timeout)
 
     def search(self, query, args = None):
         """
@@ -557,7 +569,7 @@ class Index(object):
             one is kept and others are removed.
         """
         if args == None:
-            return AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s?query=%s" % (self.url_index_name, quote(query.encode('utf8'), safe='')))
+            return AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s?query=%s" % (self.url_index_name, quote(query.encode('utf8'), safe='')), self.client.timeout)
         else:
             params = {}
             try:
@@ -570,7 +582,7 @@ class Index(object):
                 else:
                     params[k] = v
 
-            return AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s?query=%s&%s" % (self.url_index_name, quote(query.encode('utf8'), safe=''), urlencode(params)))
+            return AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s?query=%s&%s" % (self.url_index_name, quote(query.encode('utf8'), safe=''), urlencode(params)), self.client.timeout)
 
     def flatten(self, lst):
       return sum( ([x] if not isinstance(x, list) else flatten(x)
@@ -660,7 +672,7 @@ class Index(object):
                      Page is zero-based and defaults to 0. Thus, to retrieve the 10th page you need to set page=9
          @param hits_per_page: Pagination parameter used to select the number of hits per page. Defaults to 1000.
         """
-        return AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s/browse?page=%d&hitsPerPage=%d" % (self.url_index_name, page, hits_per_page))
+        return AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s/browse?page=%d&hitsPerPage=%d" % (self.url_index_name, page, hits_per_page), self.client.timeout)
 
     @deprecated
     def waitTask(self, task_id, time_before_retry = 100):
@@ -674,7 +686,7 @@ class Index(object):
         @param time_before_retry the time in milliseconds before retry (default = 100ms)
         """
         while True:
-            res = AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s/task/%d/" % (self.url_index_name, task_id))
+            res = AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s/task/%d/" % (self.url_index_name, task_id), self.client.timeout)
             if (res["status"] == "published"):
                 return res
             time.sleep(time_before_retry / 1000.)
@@ -686,7 +698,7 @@ class Index(object):
         """
         Get settings of this index
         """
-        return AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s/settings" % self.url_index_name)
+        return AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s/settings" % self.url_index_name, self.client.timeout)
 
     @deprecated
     def clearIndex(self):
@@ -695,7 +707,7 @@ class Index(object):
         """
         This function deletes the index content. Settings and index specific API keys are kept untouched.
         """
-        return AlgoliaUtils_request(self.client.headers, self.hosts, "POST", "/1/indexes/%s/clear" % self.url_index_name)
+        return AlgoliaUtils_request(self.client.headers, self.hosts, "POST", "/1/indexes/%s/clear" % self.url_index_name, self.client.timeout)
 
     @deprecated
     def setSettings(self, settings):
@@ -753,7 +765,7 @@ class Index(object):
            - optionalWords: (array of strings) Specify a list of words that should be considered as optional when found in the query.
 
          """
-        return AlgoliaUtils_request(self.client.headers, self.hosts, "PUT", "/1/indexes/%s/settings" % self.url_index_name, settings)
+        return AlgoliaUtils_request(self.client.headers, self.hosts, "PUT", "/1/indexes/%s/settings" % self.url_index_name, self.client.timeout, settings)
 
     @deprecated
     def listUserKeys(self):
@@ -762,7 +774,7 @@ class Index(object):
         """
         List all existing user keys of this index with their associated ACLs
         """
-        return AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s/keys" % self.url_index_name)
+        return AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s/keys" % self.url_index_name, self.client.timeout)
 
     @deprecated
     def getUserKeyACL(self, key):
@@ -771,7 +783,7 @@ class Index(object):
         """
         Get ACL of a user key associated to this index
         """
-        return AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s/keys/%s" % (self.url_index_name, key))
+        return AlgoliaUtils_request(self.client.headers, self.hosts, "GET", "/1/indexes/%s/keys/%s" % (self.url_index_name, key), self.client.timeout)
 
     @deprecated
     def deleteUserKey(self, key):
@@ -780,7 +792,7 @@ class Index(object):
         """
         Delete an existing user key associated to this index
         """
-        return AlgoliaUtils_request(self.client.headers, self.hosts, "DELETE", "/1/indexes/%s/keys/%s" % (self.url_index_name, key))
+        return AlgoliaUtils_request(self.client.headers, self.hosts, "DELETE", "/1/indexes/%s/keys/%s" % (self.url_index_name, key), self.client.timeout)
 
     @deprecated
     def addUserKey(self, acls, validity = 0, max_queries_per_ip_per_hour = 0, max_hits_per_query = 0):
@@ -801,7 +813,7 @@ class Index(object):
         @param max_queries_per_ip_per_hour Specify the maximum number of API calls allowed from an IP address per hour.  Defaults to 0 (no rate limit).
         @param max_hits_per_query Specify the maximum number of hits this API key can retrieve in one call. Defaults to 0 (unlimited)
         """
-        return AlgoliaUtils_request(self.client.headers, self.hosts, "POST", "/1/indexes/%s/keys" % self.url_index_name, {"acl": acls, "validity": validity, "maxQueriesPerIPPerHour": max_queries_per_ip_per_hour, "maxHitsPerQuery": max_hits_per_query} )
+        return AlgoliaUtils_request(self.client.headers, self.hosts, "POST", "/1/indexes/%s/keys" % self.url_index_name, self.client.timeout, {"acl": acls, "validity": validity, "maxQueriesPerIPPerHour": max_queries_per_ip_per_hour, "maxHitsPerQuery": max_hits_per_query} )
 
     def update_user_key(self, key, acls, validity = 0, max_queries_per_ip_per_hour = 0, max_hits_per_query = 0):
         """
@@ -819,15 +831,15 @@ class Index(object):
         @param max_queries_per_ip_per_hour Specify the maximum number of API calls allowed from an IP address per hour.  Defaults to 0 (no rate limit).
         @param max_hits_per_query Specify the maximum number of hits this API key can retrieve in one call. Defaults to 0 (unlimited)
         """
-        return AlgoliaUtils_request(self.client.headers, self.hosts, "PUT", "/1/indexes/%s/keys/%s" % (self.url_index_name, key), {"acl": acls, "validity": validity, "maxQueriesPerIPPerHour": max_queries_per_ip_per_hour, "maxHitsPerQuery": max_hits_per_query} )
+        return AlgoliaUtils_request(self.client.headers, self.hosts, "PUT", "/1/indexes/%s/keys/%s" % (self.url_index_name, key), self.client.timeout, {"acl": acls, "validity": validity, "maxQueriesPerIPPerHour": max_queries_per_ip_per_hour, "maxHitsPerQuery": max_hits_per_query} )
 
     def batch(self, request):
         """
         Send a batch request
         """
-        return AlgoliaUtils_request(self.client.headers, self.hosts, "POST", "/1/indexes/%s/batch" % self.url_index_name, request)
+        return AlgoliaUtils_request(self.client.headers, self.hosts, "POST", "/1/indexes/%s/batch" % self.url_index_name, self.client.timeout, request)
 
-def AlgoliaUtils_request(headers, hosts, method, request, body = None):
+def AlgoliaUtils_request(headers, hosts, method, request, timeout, body = None):
     """
     Util function used to send request
     """
@@ -838,7 +850,7 @@ def AlgoliaUtils_request(headers, hosts, method, request, body = None):
             if body != None:
                 obj = json.dumps(body, cls = JSONEncoderWithDatetimeAndDefaultToString)
             conn = POOL_MANAGER.connection_from_host(host, scheme = 'https')
-            answer  = conn.urlopen(method, request, headers = headers, body = obj)
+            answer  = conn.urlopen(method, request, headers = headers, body = obj, timeout=timeout)
             content = json.loads(answer.data.decode('utf-8'))
             if answer.status == 400:
                 raise AlgoliaException(content["message"])
